@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_COMMON_FIND_MATRIX_ROW_PERMUTATION_H
-#define POLYMAKE_COMMON_FIND_MATRIX_ROW_PERMUTATION_H
+#pragma once
 
 #include "polymake/GenericMatrix.h"
 #include "polymake/permutations.h"
@@ -34,7 +33,7 @@ struct matrix_elem_comparator<double> {
 };
 
 template <typename Matrix1, typename Matrix2, typename E>
-optional<Array<int>>
+optional<Array<Int>>
 find_matrix_row_permutation(const GenericMatrix<Matrix1, E>& M1, const GenericMatrix<Matrix2, E>& M2,
                             bool expect_duplicate_rows = false)
 {
@@ -47,7 +46,6 @@ find_matrix_row_permutation(const GenericMatrix<Matrix1, E>& M1, const GenericMa
 
 } }
 
-#endif // POLYMAKE_COMMON_FIND_MATRIX_ROW_PERMUTATION_H
 
 // Local Variables:
 // mode:C++
