@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_COMMON_LABELS_H
-#define POLYMAKE_COMMON_LABELS_H
+#pragma once
 
 #include "polymake/client.h"
 #include "polymake/vector"
@@ -41,7 +40,6 @@ read_labels(const BigObject& p, AnyString label_prop, Container&& labels)
 
 } }
 
-#endif // POLYMAKE_COMMON_LABELS_H
 
 // Local Variables:
 // mode:C++
