@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_GRAPH_DIJKSTRA_SHORTEST_PATH_BASE_H
-#define POLYMAKE_GRAPH_DIJKSTRA_SHORTEST_PATH_BASE_H
+#pragma once
 
 #include "polymake/Graph.h"
 #include "polymake/Heap.h"
@@ -263,7 +262,6 @@ public:
 
 } }
 
-#endif // POLYMAKE_GRAPH_DIJKSTRA_SHORTEST_PATH_BASE_H
 
 // Local Variables:
 // mode:C++
