@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -14,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_GRAPH_CONNECTED_H
-#define POLYMAKE_GRAPH_CONNECTED_H
+#pragma once
 
 #include "polymake/graph/graph_iterators.h"
 #include "polymake/IncidenceMatrix.h"
@@ -54,7 +54,7 @@ class connected_components_iterator
    : protected BFSiterator<TGraph, VisitorTag<NodeVisitor<true>>, TraversalDirectionTag<int_constant<!TGraph::is_directed>>> {
 protected:
    typedef BFSiterator<TGraph, VisitorTag<NodeVisitor<true>>, TraversalDirectionTag<int_constant<!TGraph::is_directed>>> base_t;
-   Set<int> component;
+   Set<Int> component;
 
    void fill()
    {
@@ -66,7 +66,7 @@ protected:
 
 public:
    typedef std::forward_iterator_tag iterator_category;
-   typedef Set<int> value_type;
+   typedef Set<Int> value_type;
    typedef value_type reference;
    typedef const value_type* pointer;
    typedef ptrdiff_t difference_type;
@@ -158,10 +158,9 @@ struct check_iterator_feature<polymake::graph::connected_components_iterator<TGr
 
 template <typename GraphRef>
 class generic_of_GraphComponents<GraphRef, polymake::graph::connected_components_iterator>
-   : public GenericSet< GraphComponents<GraphRef, polymake::graph::connected_components_iterator>, Set<int>, operations::cmp > {};
+   : public GenericSet< GraphComponents<GraphRef, polymake::graph::connected_components_iterator>, Set<Int>, operations::cmp > {};
 }
 
-#endif // POLYMAKE_GRAPH_CONNECTED_H
 
 // Local Variables:
 // mode:C++
