@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -14,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_IDEAL_SINGULAR_IDEAL_H
-#define POLYMAKE_IDEAL_SINGULAR_IDEAL_H
+#pragma once
 
 
 // polymake includes
@@ -61,8 +61,8 @@ public:
 
    virtual Array<Polynomial<>> polynomials() const = 0;
    
-   static SingularIdeal_wrap* create(const Array<Polynomial<>>& gens, const Vector<int>& order);
-   static SingularIdeal_wrap* create(const Array<Polynomial<>>& gens, const Matrix<int>& order);
+   static SingularIdeal_wrap* create(const Array<Polynomial<>>& gens, const Vector<Int>& order);
+   static SingularIdeal_wrap* create(const Array<Polynomial<>>& gens, const Matrix<Int>& order);
    static SingularIdeal_wrap* create(const Array<Polynomial<>>& gens, const std::string& order);
 
 //   static SingularIdeal_wrap* quotient(const SingularIdeal_wrap* I, const SingularIdeal_wrap* J);
@@ -130,9 +130,9 @@ public:
       return SingularIdeal(singIdeal->saturation(rhs));
    }
 
-   perl::ListReturn primary_decomposition() const {
+   ListReturn primary_decomposition() const {
       Array<SingularIdeal_wrap*> pd = singIdeal->primary_decomposition();
-      perl::ListReturn result;
+      ListReturn result;
       for (auto id = entire(pd); !id.at_end(); ++id) {
          result << SingularIdeal(*id);
       }
@@ -143,7 +143,7 @@ public:
       return singIdeal->polynomials();
    }
    
-   friend perl::Object quotient(perl::Object I, perl::Object J);
+   friend BigObject quotient(BigObject I, BigObject J);
 };
 /*template<>
 class TermOrder<Matrix<Integer> >{
@@ -206,4 +206,3 @@ public:
 } // end namespace ideal
 } // end namespace polymake
 
-#endif
