@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_INTERNAL_SHARED_OBJECT_H
-#define POLYMAKE_INTERNAL_SHARED_OBJECT_H
+#pragma once
 
 #include "polymake/internal/iterators.h"
 #include "polymake/internal/operations.h"
@@ -1298,7 +1297,6 @@ namespace std {
 
 #include "polymake/internal/alias.h"
 
-#endif // POLYMAKE_INTERNAL_SHARED_OBJECT_H
 
 // Local Variables:
 // mode:C++
