@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_VALMAT_CHECK_VALUATED_AXIOMS_H
-#define POLYMAKE_VALMAT_CHECK_VALUATED_AXIOMS_H
+#pragma once
 
 #include "polymake/Rational.h"
 #include "polymake/list"
@@ -31,18 +30,18 @@ namespace polymake { namespace matroid {
 /**
  * @brief Finds the valuation of a subset in a valuated matroid. If its a basis, it returns
  * the corresponding valuation. Otherwise, it returns tropical zero.
- * @param Array<Set<int> > bases. The list of bases.
+ * @param Array<Set<Int>> bases. The list of bases.
  * @param Vector<TropicalNumber<Addition,Scalar> > valuation. The valuations.
- * @param Set<int> s. The set whose valuation should be computed.
- * @return int. The valuation of s.
+ * @param Set<Int> s. The set whose valuation should be computed.
+ * @return Int The valuation of s.
  */
 template <typename Addition, typename Scalar>
-TropicalNumber<Addition,Scalar> find_valuation(const Array<Set<int>>& bases,
-                                               const Vector<TropicalNumber<Addition,Scalar>>& valuation,
-                                               const Set<int>& s)
+TropicalNumber<Addition,Scalar> find_valuation(const Array<Set<Int>>& bases,
+                                               const Vector<TropicalNumber<Addition, Scalar>>& valuation,
+                                               const Set<Int>& s)
 {
-  for (int b = 0; b < bases.size(); b++) {
-    Set<int> inter = bases[b] * s;
+  for (Int b = 0; b < bases.size(); ++b) {
+    Set<Int> inter = bases[b] * s;
       if (inter.size() == bases[b].size() && inter.size() == s.size()) {
         return valuation[b];
       }
@@ -53,9 +52,9 @@ TropicalNumber<Addition,Scalar> find_valuation(const Array<Set<int>>& bases,
 // -------------------------------------------
   
 template <typename Addition, typename Scalar>
-bool check_valuated_basis_axioms(const Array<Set<int>>& bases,
+bool check_valuated_basis_axioms(const Array<Set<Int>>& bases,
                                  const Vector<TropicalNumber<Addition, Scalar>>& valuation,
-                                 perl::OptionSet options)
+                                 OptionSet options)
 {
   bool verbose = options["verbose"];
   // Sanity check
@@ -69,16 +68,16 @@ bool check_valuated_basis_axioms(const Array<Set<int>>& bases,
   }
 
   // Check basis exchange axiom
-  for (int i = 0; i < bases.size(); i++) {
-    for (int j = 0; j < bases.size(); j++) {
+  for (Int i = 0; i < bases.size(); ++i) {
+    for (Int j = 0; j < bases.size(); ++j) {
       if (i != j) {
-        Set<int> iwithoutj = bases[i] - bases[j];
-        Set<int> jwithouti = bases[j] - bases[i];
+        Set<Int> iwithoutj = bases[i] - bases[j];
+        Set<Int> jwithouti = bases[j] - bases[i];
         for (auto u = entire(iwithoutj); !u.at_end(); ++u) {
           bool found_one = false;
           for (auto v = entire(jwithouti); !v.at_end(); ++v) {
-            Set<int> B1 = bases[i] - *u + *v;
-            Set<int> B2 = bases[j] - *v + *u;
+            Set<Int> B1 = bases[i] - *u + *v;
+            Set<Int> B2 = bases[j] - *v + *u;
             TropicalNumber<Addition,Scalar> wbi = valuation[i];
             TropicalNumber<Addition,Scalar> wbj = valuation[j];
             TropicalNumber<Addition,Scalar> wB1 = find_valuation(bases,valuation,B1);
@@ -103,14 +102,14 @@ bool check_valuated_basis_axioms(const Array<Set<int>>& bases,
 // -------------------------------------------
   
 template <typename Addition, typename Scalar>
-bool check_valuated_circuit_axioms(const Matrix<TropicalNumber<Addition,Scalar>>& valuations, perl::OptionSet options)
+bool check_valuated_circuit_axioms(const Matrix<TropicalNumber<Addition,Scalar>>& valuations, OptionSet options)
 {
   bool verbose = options["verbose"];
   // First, we compute supports
-  Vector<Set<int>> supports;
-  for (int r = 0; r < valuations.rows(); r++) {
-    Set<int> supp;
-    for (int i = 0; i < valuations.cols(); i++) {
+  Vector<Set<Int>> supports;
+  for (Int r = 0; r < valuations.rows(); ++r) {
+    Set<Int> supp;
+    for (Int i = 0; i < valuations.cols(); ++i) {
       if (TropicalNumber<Addition,Scalar>::zero() != valuations(r,i)) supp += i;
     }
     // Check for the empty set
@@ -122,9 +121,9 @@ bool check_valuated_circuit_axioms(const Matrix<TropicalNumber<Addition,Scalar>>
   } //END compute supports
 
   // Check for equal or contained supports.
-  for (int i = 0; i < supports.dim(); ++i) {
-    for (int j = i+1; j < supports.dim(); ++j) {
-      Set<int> inter = supports[i] * supports[j];
+  for (Int i = 0; i < supports.dim(); ++i) {
+    for (Int j = i+1; j < supports.dim(); ++j) {
+      Set<Int> inter = supports[i] * supports[j];
       if (inter.size() == supports[i].size() || inter.size() == supports[j].size()) {
         if (verbose) cout << "Circuits " << i << " and " << j << " have contained supports." << endl;
         return false;
@@ -137,11 +136,11 @@ bool check_valuated_circuit_axioms(const Matrix<TropicalNumber<Addition,Scalar>>
   // set X' = (Y_u / X_u) * X (all ops are tropical), so X'_u = Y_u. Then there exists
   // a circuit Z such that u notin supp Z, v in supp Z and 
   // X' + Y + (X_v / Z_v)*Z = X' + Y.
-  for (int x = 0; x < valuations.rows(); ++x) {
-    for (int y = 0; y < valuations.rows(); ++y) {
+  for (Int x = 0; x < valuations.rows(); ++x) {
+    for (Int y = 0; y < valuations.rows(); ++y) {
       if (x != y) {
-        Set<int> suppboth = supports[x] * supports[y];
-        Set<int> supponlyx = supports[x] - supports[y];
+        Set<Int> suppboth = supports[x] * supports[y];
+        Set<Int> supponlyx = supports[x] - supports[y];
         for (auto u = entire(suppboth); !u.at_end(); u++) {
           for (auto v = entire(supponlyx); !v.at_end(); v++) {
             bool found_one = false;
@@ -149,7 +148,7 @@ bool check_valuated_circuit_axioms(const Matrix<TropicalNumber<Addition,Scalar>>
             Vector<TropicalNumber<Addition,Scalar>> xprime = valuations.row(x);
             xprime *= valuations(y,*u) / valuations(x,*u);
             // Go through circuits that don't have u in its support, but do contain v
-            for (int z = 0; z < supports.dim(); ++z) {
+            for (Int z = 0; z < supports.dim(); ++z) {
               if (!supports[z].contains(*u) && supports[z].contains(*v)) {
                 // Normalize Z such that it agrees with X' on v.
                 Vector<TropicalNumber<Addition, Scalar>> zprime = valuations.row(z);
@@ -180,4 +179,9 @@ bool check_valuated_circuit_axioms(const Matrix<TropicalNumber<Addition,Scalar>>
 
 } }
 
-#endif
+
+// Local Variables:
+// mode:C++
+// c-basic-offset:3
+// indent-tabs-mode:nil
+// End:
