@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_ACCURATERATIONAL_H
-#define POLYMAKE_ACCURATERATIONAL_H
+#pragma once
 
 #include "polymake/Rational.h"
 #include <mpfr.h>
@@ -1793,7 +1792,6 @@ namespace polymake {
 using pm::AccurateFloat;
 }
 
-#endif // POLYMAKE_ACCURATERATIONAL_H
 
 // Local Variables:
 // mode:C++
