@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_POLYTOPE_GENERIC_MILP_CLIENT_H
-#define POLYMAKE_POLYTOPE_GENERIC_MILP_CLIENT_H
+#pragma once
 
 #include "polymake/client.h"
 #include "polymake/linalg.h"
@@ -26,7 +25,7 @@ namespace polymake { namespace polytope {
 
 
 template <typename Scalar>
-void store_MILP_Solution(perl::Object& p, perl::Object& milp, bool maximize, const MILP_Solution<Scalar>& S)
+void store_MILP_Solution(BigObject& p, BigObject& milp, bool maximize, const MILP_Solution<Scalar>& S)
 {
    if (S.status == LP_status::valid) {
       milp.take(maximize ? Str("MAXIMAL_VALUE") : Str("MINIMAL_VALUE")) << S.objective_value;
@@ -42,13 +41,13 @@ void store_MILP_Solution(perl::Object& p, perl::Object& milp, bool maximize, con
 
 
 template <typename Scalar, typename Solver>
-void generic_milp_client(perl::Object& p, perl::Object& milp, bool maximize, const Solver& MILP_solver)
+void generic_milp_client(BigObject& p, BigObject& milp, bool maximize, const Solver& MILP_solver)
 {
    std::string H_name;
    const Matrix<Scalar> H = p.give("FACETS | INEQUALITIES"),
                         E = p.lookup("AFFINE_HULL | EQUATIONS");
    const Vector<Scalar> Obj = milp.give("LINEAR_OBJECTIVE");
-   Set<int> integerVariables;
+   Set<Int> integerVariables;
    milp.lookup("INTEGER_VARIABLES") >> integerVariables;
    // Default to taking all variables as integer, since otherwise one should be
    // using a LP solver.
@@ -67,7 +66,6 @@ void generic_milp_client(perl::Object& p, perl::Object& milp, bool maximize, con
 
 } }
 
-#endif // POLYMAKE_POLYTOPE_GENERIC_MILP_CLIENT_H
 
 // Local Variables:
 // mode:C++
