@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -14,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_POLYTOPE_PPL_INTERFACE_H
-#define POLYMAKE_POLYTOPE_PPL_INTERFACE_H
+#pragma once
 
 #include "polymake/polytope/solve_LP.h"
 #include "polymake/polytope/convex_hull.h"
@@ -41,7 +41,6 @@ public:
 
 } } }
 
-#endif // POLYMAKE_POLYTOPE_PPL_INTERFACE_H
 
 // Local Variables:
 // mode:C++
