@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -14,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_POLYTOPE_TRANSFORM_H
-#define POLYMAKE_POLYTOPE_TRANSFORM_H
+#pragma once
 
 #include "polymake/client.h"
 #include "polymake/IncidenceMatrix.h"
@@ -32,7 +32,7 @@ namespace polymake { namespace polytope {
     @param tau transformation matrix
 */
 template <typename TransMatrix> inline
-void transform_section(perl::Object& p_out, perl::Object& p_in,
+void transform_section(BigObject& p_out, BigObject& p_in,
                        const AnyString& section_req,
                        const GenericMatrix<TransMatrix>& tau)
 {
@@ -51,12 +51,12 @@ void transform_section(perl::Object& p_out, perl::Object& p_in,
     @param tau transformation matrix for points
 */
 template <typename Scalar, typename TransMatrix>
-perl::Object transform(perl::Object p_in, const GenericMatrix<TransMatrix>& tau,
-                       bool store_reverse_transformation=true)
+BigObject transform(BigObject p_in, const GenericMatrix<TransMatrix>& tau,
+                    bool store_reverse_transformation = true)
 {
-   const auto tau_inv=inv(tau);
+   const auto tau_inv = inv(tau);
 
-   perl::Object p_out("Polytope", mlist<Scalar>());
+   BigObject p_out("Polytope", mlist<Scalar>());
 
    transform_section(p_out, p_in, "VERTICES | POINTS", tau);
    transform_section(p_out, p_in, "LINEALITY_SPACE | INPUT_LINEALITY", tau);
@@ -87,7 +87,6 @@ perl::Object transform(perl::Object p_in, const GenericMatrix<TransMatrix>& tau,
 
 } } // end namespace polymake
 
-#endif // POLYMAKE_POLYTOPE_TRANSFORM_H
 
 // Local Variables:
 // mode:C++
