@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2021
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -38,8 +38,8 @@ public:
    // Each DecoratedEdge that appears in the BFS will get an unique index via this map.
    using EdgeMap = Map<Int, DecoratedEdge>;
 
-   using DoublyConnectedEdgeList = graph::DoublyConnectedEdgeList;
-   using HalfEdge = DoublyConnectedEdgeList::HalfEdge;
+   using DoublyConnectedEdgeList = graph::dcel::DoublyConnectedEdgeList;
+   using HalfEdge = graph::dcel::HalfEdge;
 
 private:
    // The graph we want to iterate through, built during iteration.
