@@ -18,7 +18,7 @@
 	Copyright (C) 2011 - 2015, Simon Hampe <simon.hampe@googlemail.com>
 
 	---
-	Copyright (c) 2016-2019
+	Copyright (c) 2016-2022
 	Ewgenij Gawrilow, Michael Joswig, and the polymake team
 	Technische Universität Berlin, Germany
 	https://polymake.org
@@ -26,13 +26,14 @@
 	Functions to create special morphisms.
 	*/
 
-#ifndef POLYMAKE_ATINT_MORPHISM_SPECIAL_H
-#define POLYMAKE_ATINT_MORPHISM_SPECIAL_H
+#pragma once
 
 #include "polymake/client.h"
 #include "polymake/Matrix.h"
 #include "polymake/Rational.h"
 #include "polymake/Vector.h"
+#include "polymake/SparseVector.h"
+#include "polymake/SparseMatrix.h"
 #include "polymake/Set.h"
 #include "polymake/Map.h"
 
@@ -40,14 +41,14 @@ namespace polymake { namespace tropical {
 
 // Documentation see perl wrapper
 template <typename Addition>
-perl::Object evaluation_map(int n, const Matrix<Rational>& delta, int i)
+BigObject evaluation_map(Int n, const Matrix<Rational>& delta, Int i)
 {
   if (n <= 0 || delta.rows() <= 0 || i <= 0 || i > n) {
     throw std::runtime_error("Cannot create evaluation map: Invalid parameters");
   }
 
   // Dimension of the target space
-  int r = delta.cols()-1;
+  Int r = delta.cols()-1;
 
   // We create the map on the 
   // The matrix of ev_i is pr(R^r) + G*pr(M_0,N), where
@@ -59,12 +60,12 @@ perl::Object evaluation_map(int n, const Matrix<Rational>& delta, int i)
   // row of delta). 
 
   // Projection matrices
-  int N = n + delta.rows();
-  int modulidim = (N*(N-3))/2+1; 
+  Int N = n + delta.rows();
+  Int modulidim = (N*(N-3))/2+1; 
 
   // The projection to R^r needs the first coordinate as homogenizing coordinate.
-  Matrix<Rational> projR = Matrix<Rational>(r,modulidim) | unit_matrix<Rational>(r);
-  projR = unit_vector<Rational>(projR.cols(),0) / projR;
+  const Matrix<Rational> projR = unit_vector<Rational>(r+modulidim, 0) /
+                                 (zero_matrix<Rational>(r, modulidim) | unit_matrix<Rational>(r));
 
   Matrix<Rational> projM = unit_matrix<Rational>(modulidim) | Matrix<Rational>(modulidim,r);
 
@@ -73,34 +74,31 @@ perl::Object evaluation_map(int n, const Matrix<Rational>& delta, int i)
     // First we create the edge index matrix E(i,j) that contains at element i,j 
     // the edge index of edge (i,j)
     // in the complete graph on N-1 nodes (in lexicographic order).
-    int nextindex = 0;
-    Matrix<int> E(N-1,N-1);
-    for (int k = 0; k < N-2; ++k) {
-      for (int j = k+1; j < N-1; ++j) {
+    Int nextindex = 0;
+    Matrix<Int> E(N-1,N-1);
+    for (Int k = 0; k < N-2; ++k) {
+      for (Int j = k+1; j < N-1; ++j) {
         E(k,j) = nextindex;
         E(j,k) = nextindex;
         ++nextindex;
       }
     }	
     // Create index map (i,j) -> index(i,j)
-    int evalIndex = delta.rows() + i-1;
+    Int evalIndex = delta.rows() + i-1;
     // Set all rows corr. to coordinate (n+i,k) to -  v_k
-    for (int k = 0; k < delta.rows(); ++k) {
+    for (Int k = 0; k < delta.rows(); ++k) {
       G.col(E(evalIndex,k)) = Addition::orientation() * delta.row(k);
     }
   }
 
   Matrix<Rational> map_matrix = projR + G*projM;
 
-  perl::Object morphism("Morphism", mlist<Addition>());
-  morphism.take("MATRIX") << map_matrix;
-
-  return morphism;
+  return BigObject("Morphism", mlist<Addition>(), "MATRIX", map_matrix);
 }
 
 // Documentation see perl wrapper
 template <typename Addition>
-perl::Object evaluation_map_d(int n, int r, int d, int i)
+BigObject evaluation_map_d(Int n, Int r, Int d, Int i)
 {
   if (n <= 0 || r <= 0 || d <= 0 || i <= 0 || i > n) {
     throw std::runtime_error("Cannot create evaluation map: Invalid parameters");
@@ -108,8 +106,8 @@ perl::Object evaluation_map_d(int n, int r, int d, int i)
 
   // Create standard d-fold direction matrix
   Matrix<Rational> delta(0,r+1);
-  for (int k = 0; k <= r; ++k) {
-    for (int j = 1; j <= d; ++j) {
+  for (Int k = 0; k <= r; ++k) {
+    for (Int j = 1; j <= d; ++j) {
       delta /= unit_vector<Rational>(r+1,k);
     }
   }
@@ -119,12 +117,12 @@ perl::Object evaluation_map_d(int n, int r, int d, int i)
 
 // Documentation see perl wrapper
 template <typename Addition>
-perl::Object projection_map(int n, const Set<int>& coords)
+BigObject projection_map(Int n, const Set<Int>& coords)
 {
   // Create matrix
   Matrix<Rational> proj_matrix(coords.size(), n+1);
-  int image_index = 0;
-  for (const int c : coords) {
+  Int image_index = 0;
+  for (const Int c : coords) {
     if (c > n) {
       throw std::runtime_error("Cannot create projection: Image dimension larger than domain dimension");
     }
@@ -132,15 +130,12 @@ perl::Object projection_map(int n, const Set<int>& coords)
     ++image_index;
   }
 
-  perl::Object result("Morphism", mlist<Addition>());
-  result.take("MATRIX") << proj_matrix;
-
-  return result;
+  return BigObject("Morphism", mlist<Addition>(), "MATRIX", proj_matrix);
 }
 
 // Documentation see perl wrapper
 template <typename Addition>
-perl::Object projection_map_default(int n, int m)
+BigObject projection_map_default(Int n, Int m)
 {
   if (m > n) {
     throw std::runtime_error("Cannot create projection: Image dimension larger than domain dimension");
@@ -150,7 +145,7 @@ perl::Object projection_map_default(int n, int m)
 
 // Documentation see perl wrapper
 template <typename Addition>
-perl::Object forgetful_map(int n, const Set<int>& leaves_to_forget)
+BigObject forgetful_map(Int n, const Set<Int>& leaves_to_forget)
 {
   // First check, that the leaves are in (1,..,n)
   if (leaves_to_forget.front() < 1 || leaves_to_forget.back() > n) {
@@ -158,28 +153,24 @@ perl::Object forgetful_map(int n, const Set<int>& leaves_to_forget)
   }
 
   // Compute domain and image dimension
-  int domain_dim = (n*(n-3))/2 +1;
-  int small_n = n - leaves_to_forget.size();
-  int image_dim = (small_n*(small_n - 3))/2 + 1;
+  Int domain_dim = (n*(n-3))/2 +1;
+  Int small_n = n - leaves_to_forget.size();
+  Int image_dim = (small_n*(small_n - 3))/2 + 1;
 
   // Check if we forget so many leaves that we get the zero map
   if (small_n <= 3) {
-    perl::Object result("Morphism", mlist<Addition>());
-    result.take("MATRIX") << Matrix<Rational>(small_n == 3 ? 1 : 0, domain_dim);
-    return result;
+    return BigObject("Morphism", mlist<Addition>(), "MATRIX", Matrix<Rational>(small_n == 3 ? 1 : 0, domain_dim));
   }
   // Check if we don't forget anything at all
   if (leaves_to_forget.size() == 0) {
-    perl::Object result("Morphism", mlist<Addition>());
-    Matrix<Rational> um = unit_matrix<Rational>(domain_dim);
-    result.take("MATRIX") << um;
-    return result;
+    return BigObject("Morphism", mlist<Addition>(),
+                     "MATRIX", unit_matrix<Rational>(domain_dim));
   }
 
   // Prepare map mapping remaining leaves to {1,..,n-|leaves_to_forget|}
-  Map<int,int> remaining_map;
-  int next_index = 1;
-  for (int i = 1; i <= n; ++i) {
+  Map<Int, Int> remaining_map;
+  Int next_index = 1;
+  for (Int i = 1; i <= n; ++i) {
     if (!leaves_to_forget.contains(i)) {
       remaining_map[i] = next_index;
       ++next_index;
@@ -188,10 +179,10 @@ perl::Object forgetful_map(int n, const Set<int>& leaves_to_forget)
 
   // This maps each edge (i,j) in the complete graph on small_n-1 vertices to an
   // index in 0 .. N. Each such edge is a flat, whose vector is (minus) a unit vector
-  Map<std::pair<int,int>, int> unit_edges;
-  int unit_index = 0;
-  for (int i = 1; i < small_n-1; ++i) {
-    for (int j = i+1; j < small_n; ++j) {
+  Map<std::pair<Int, Int>, Int> unit_edges;
+  Int unit_index = 0;
+  for (Int i = 1; i < small_n-1; ++i) {
+    for (Int j = i+1; j < small_n; ++j) {
       unit_edges[std::make_pair(i,j)] = unit_index;
       ++unit_index;
     }
@@ -202,15 +193,15 @@ perl::Object forgetful_map(int n, const Set<int>& leaves_to_forget)
 
   // Compute image of each unit vector e_i, correpsonding to all v_{k,l} (- v_kl for Max)
   // with 1 <= k < l < n
-  for (int k = 1; k < n-1; ++k) {
-    for (int l = k+1; l < n; ++l) {
+  for (Int k = 1; k < n-1; ++k) {
+    for (Int l = k+1; l < n; ++l) {
       // If klset contains forgotten leaves, the ray is mapped to zero
       if (leaves_to_forget.contains(k) || leaves_to_forget.contains(l)) {
         ffm |= zero_vector<Rational>(image_dim);
       } else {
         // First we compute the new number of the leaves in the image
-        int newk = remaining_map[k];
-        int newl = remaining_map[l];
+        Int newk = remaining_map[k];
+        Int newl = remaining_map[l];
 
         // The easy case is that newl is not the new maximal leaf
         if (newl < small_n) {
@@ -220,13 +211,13 @@ perl::Object forgetful_map(int n, const Set<int>& leaves_to_forget)
         else {
           // If the new set contains the maximal leaf, we have to take the complement to 
           // compute the moduli coordinates
-          Set<int> complement = (sequence(1,small_n) - newk) - newl;
+          Set<Int> complement = (sequence(1,small_n) - newk) - newl;
           // Compute moduli coordinates as usual: Sum up over all flat vectors of the 
           // 1-edge flats, i.e. all pairs contained in complement
           Vector<Rational> ray_image(image_dim);
-          Vector<int> slist(complement);
-          for (int i = 0; i < slist.dim(); ++i) {
-            for (int j = i+1; j < slist.dim(); ++j) {
+          Vector<Int> slist(complement);
+          for (Int i = 0; i < slist.dim(); ++i) {
+            for (Int j = i+1; j < slist.dim(); ++j) {
               // If its the last edge (small_n-2, small_n-1), we add the ones_vector, otherwise, 
               // we substract the corr. unit_vector
               ray_image[ unit_edges[std::make_pair(slist[i], slist[j])] ] += 1;
@@ -241,13 +232,8 @@ perl::Object forgetful_map(int n, const Set<int>& leaves_to_forget)
     } //END iterate l
   } //END iterate k
 
-  // Return result
-  perl::Object result("Morphism", mlist<Addition>());
-  result.take("MATRIX") << ffm;
-
-  return result;
+  return BigObject("Morphism", mlist<Addition>(), "MATRIX", ffm);
 }
 
 } }
 
-#endif
