@@ -18,15 +18,14 @@
    Copyright (C) 2011 - 2015, Simon Hampe <simon.hampe@googlemail.com>
 
    ---
-   Copyright (c) 2016-2020
+   Copyright (c) 2016-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
 
 */
 
-#ifndef POLYMAKE_ATINT_STAR_H
-#define POLYMAKE_ATINT_STAR_H
+#pragma once
 
 #include "polymake/Rational.h"
 #include "polymake/Matrix.h"
@@ -76,4 +75,3 @@ computeStar(const Vector<Rational>& point,
 
 } }
 
-#endif
