@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2015
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -20,11 +21,11 @@
 
 namespace polymake { namespace graph {
 
-void bipartite_signature(perl::Object p)
+void bipartite_signature(BigObject p)
 {
    const Graph<> G=p.give("ADJACENCY");
-   const int sign=bipartite_sign(G);
-   p.take("BIPARTITE") << (sign>=0);
+   const Int sign = bipartite_sign(G);
+   p.take("BIPARTITE") << (sign >= 0);
    p.take("SIGNATURE") << sign;
 }
 
