@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2015
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -20,13 +21,13 @@
 
 namespace polymake { namespace graph {
 
-void degree_sequence(perl::Object G)
+void degree_sequence(BigObject G)
 {
-   Map<int,int> ds;
-   int sum_degree(0);
+   Map<Int, Int> ds;
+   Int sum_degree = 0;
    const Graph<> g = G.give("ADJACENCY");
-   for (int i=0; i<g.nodes(); ++i) {
-      const int deg = g.out_degree(i);
+   for (Int i = 0; i < g.nodes(); ++i) {
+      const Int deg = g.out_degree(i);
       ds[deg]++;
       sum_degree += deg;
    }
@@ -34,7 +35,7 @@ void degree_sequence(perl::Object G)
    G.take("AVERAGE_DEGREE") << Rational(sum_degree, g.nodes());
 }
 
-Function4perl(&degree_sequence, "degree_sequence($)");
+Function4perl(&degree_sequence, "degree_sequence(Graph)");
 
 } }
 
