@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -27,7 +27,7 @@ namespace polymake { namespace group {
 namespace {
 
 void perl_action_from_group_impl(const PermlibGroup& permlib_group, 
-                                 perl::Object action,
+                                 BigObject action,
                                  const std::string& name,
                                  const std::string& description) 
 {
@@ -35,9 +35,9 @@ void perl_action_from_group_impl(const PermlibGroup& permlib_group,
    BSGSSchreierExport exp;
    BSGSSchreierData* data = exp.exportData(*permlib_group.get_permlib_group());
       
-   Array<Array<int>> transversals = arrays2PolymakeArray(data->transversals, data->baseSize, data->n);
-   Array<Array<int>> sgs = arrays2PolymakeArray(data->sgs, data->sgsSize, data->n);
-   Array<int> base = array2PolymakeArray(data->base, data->baseSize);
+   Array<Array<Int>> transversals = arrays2PolymakeArray(data->transversals, data->baseSize, data->n);
+   Array<Array<Int>> sgs = arrays2PolymakeArray(data->sgs, data->sgsSize, data->n);
+   Array<Int> base = array2PolymakeArray(data->base, data->baseSize);
    delete data;
 
    action.take("STRONG_GENERATORS") << sgs;
@@ -50,29 +50,29 @@ void perl_action_from_group_impl(const PermlibGroup& permlib_group,
 
 } // end anonymous namespace
 
-Array<Array<int>>
+Array<Array<Int>>
 generators_from_permlib_group(const PermlibGroup& permlib_group)
 {
    using namespace permlib::exports;
    BSGSSchreierExport exp;
    BSGSSchreierData* data = exp.exportData(*permlib_group.get_permlib_group());
-   Array<Array<int>> sgs = arrays2PolymakeArray(data->sgs, data->sgsSize, data->n);
+   Array<Array<Int>> sgs = arrays2PolymakeArray(data->sgs, data->sgsSize, data->n);
    delete data;
    return sgs;
 }
 
-perl::Object perl_action_from_group(const PermlibGroup& permlib_group,
+BigObject perl_action_from_group(const PermlibGroup& permlib_group,
                                     const std::string& name,
                                     const std::string& description)
 {
-   perl::Object pa("group::PermutationAction");
+   BigObject pa("group::PermutationAction");
    perl_action_from_group_impl(permlib_group, pa, name, description);
    return pa;
 }
 
-void perl_action_from_generators(const Array<Array<int>>& generators,
-                                 perl::Object action,
-                                 perl::OptionSet options)
+void perl_action_from_generators(const Array<Array<Int>>& generators,
+                                 BigObject action,
+                                 OptionSet options)
 {
    const std::string name = options["name"];
    const std::string description = options["description"];
@@ -80,83 +80,80 @@ void perl_action_from_generators(const Array<Array<int>>& generators,
 }
 
 
-perl::Object perl_group_from_group(const PermlibGroup& permlib_group,
+BigObject perl_group_from_group(const PermlibGroup& permlib_group,
                                    const std::string& name,
                                    const std::string& description)
 {
-   perl::Object G("group::Group");
+   BigObject G("group::Group");
    G.take("PERMUTATION_ACTION") << perl_action_from_group(permlib_group, name, description);
    return G;
 }
   
 
 
-PermlibGroup group_from_perl_action(perl::Object action)
+PermlibGroup group_from_perl_action(BigObject action)
 {
    using namespace permlib::exports;
 
-   Array<int> base;
-   Array<Array<int>> sgs;
-   Array<Array<int>> transversals;
-   int n = 0;
+   Array<Int> base;
+   Array<Array<Int>> sgs;
+   Array<Array<Int>> transversals;
+   Int n = 0;
       
    // If group has a BSGS in perl, use it for C++.
    // Otherwise, compute BSGS from scratch (ie. from GENERATORS).
    if ((action.lookup("BASE") >> base) && 
        (action.lookup("STRONG_GENERATORS") >> sgs) &&
-       (action.lookup("TRANSVERSALS") >> transversals))
-      {
-         if ( !(action.lookup("DEGREE") >> n) ) {
-            if (sgs.size() > 0) {
-               n = sgs[0].size();
-            } else {
-               throw std::runtime_error("group_from_perl_action: could not compute DEGREE for trivial group");
-            }
+       (action.lookup("TRANSVERSALS") >> transversals)) {
+      if (!(action.lookup("DEGREE") >> n)) {
+         if (sgs.size() > 0) {
+            n = sgs[0].size();
+         } else {
+            throw std::runtime_error("group_from_perl_action: could not compute DEGREE for trivial group");
          }
-         
-         BSGSSchreierImport imp;
-         BSGSSchreierData data;
+      }
 
-         data.n = n;
-         data.baseSize = base.size();
-         data.sgsSize = sgs.size();
-         
-         data.base = polymakeArray2Array<permlib::dom_int>(base);
-         data.sgs = polymakeArray2Arrays<permlib::dom_int>(sgs);
-         data.transversals = polymakeArray2Arrays<int>(transversals);
-         
-         boost::shared_ptr<permlib::PermutationGroup> perm_group(imp.importData(&data));
-         return PermlibGroup(perm_group);
-      }
-   else
-      {
-         Array<Array<int>> generators;
-         action.give("STRONG_GENERATORS | GENERATORS") >> generators;
-         return PermlibGroup(generators);
-      }
+      BSGSSchreierImport imp;
+      BSGSSchreierData data;
+
+      data.n = permlib::safe_to_dom_int(n);
+      data.baseSize = permlib::safe_to_dom_int(base.size());
+      data.sgsSize = permlib::safe_to_dom_int(sgs.size());
+
+      data.base = polymakeArray2Array<permlib::dom_int>(base);
+      data.sgs = polymakeArray2Arrays<permlib::dom_int>(sgs);
+      data.transversals = polymakeArray2Arrays<int>(transversals);
+
+      boost::shared_ptr<permlib::PermutationGroup> perm_group(imp.importData(&data));
+      return PermlibGroup(perm_group);
+   } else {
+      Array<Array<Int>> generators;
+      action.give("STRONG_GENERATORS | GENERATORS") >> generators;
+      return PermlibGroup(generators);
+   }
 }
 
 
 /* helpers */  
 
-perl::Object group_from_permlib_cyclic_notation(const Array<std::string>& cyc_not, int degree)
+BigObject group_from_permlib_cyclic_notation(const Array<std::string>& cyc_not, Int degree)
 {
-   Array<Array<int>> parsed_generators;
+   Array<Array<Int>> parsed_generators;
    const PermlibGroup permlib_group = PermlibGroup::permgroup_from_cyclic_notation(cyc_not, degree, parsed_generators);
-   perl::Object action(perl_action_from_group(permlib_group));
+   BigObject action(perl_action_from_group(permlib_group));
    action.take("GENERATORS") << parsed_generators;      
    action.take("DEGREE") << degree;
-   perl::Object G("Group");
+   BigObject G("Group");
    G.take("PERMUTATION_ACTION") << action;
    return G;
 }
     
-std::string action_to_cyclic_notation(perl::Object action)
+std::string action_to_cyclic_notation(BigObject action)
 {
-   Array<Array<int>> generators;
+   Array<Array<Int>> generators;
    action.give("STRONG_GENERATORS | GENERATORS") >> generators;
    std::stringstream ss;
-   int count = generators.size()-1;
+   Int count = generators.size()-1;
    for (auto perm = entire(generators); !perm.at_end(); ++perm) {
       const permlib::Permutation gen(perm->begin(), perm->end());
       ss << gen;
@@ -176,7 +173,8 @@ std::string action_to_cyclic_notation(perl::Object action)
 
    //action on DOMAIN
    //constructor for permlib::PermutationGroup(Transversals,Base,SG) would be good!!!
-Array<hash_set<int>> orbits_of_action(perl::Object action) {
+Array<hash_set<Int>> orbits_of_action(BigObject action)
+{
    return group_from_perl_action(action).orbits();
 }
 
@@ -191,7 +189,7 @@ namespace {
 
   The main use cases are:
 
-     DomainType =      Set<int>,
+     DomainType =      Set<Int>,
      DomainContainer = IncidenceMatrix<>,
      Action =          SetOfIndicesAction<permlib::Permutation>
 
@@ -201,25 +199,26 @@ namespace {
      DomainContainer = GenericMatrix<MatrixTop, Scalar>
      Action =          CoordinateAction<permlib::Permutation,Scalar>
 */    
-template<typename DomainType, typename DomainContainer, typename Action>
-Array<hash_set<int>>
-orbits_of_induced_action_impl(perl::Object action, const DomainContainer& container) {
+template <typename DomainType, typename DomainContainer, typename Action>
+Array<hash_set<Int>>
+orbits_of_induced_action_impl(BigObject action, const DomainContainer& container)
+{
    const PermlibGroup group_of_cone = group_from_perl_action(action);
-   hash_map<DomainType, int> index_of;
+   hash_map<DomainType, Int> index_of;
    std::vector<DomainType> domain_list;
    domain_list.reserve(container.rows());
-   int i(0);
-   for (auto cit=entire(rows(container)); !cit.at_end(); ++cit, ++i) {	
+   Int i = 0;
+   for (auto cit = entire(rows(container)); !cit.at_end(); ++cit, ++i) {	
       domain_list.push_back(*cit);
       index_of[*cit] = i;
    }
 
    const auto olist = permlib::orbits<DomainType, Action>(*(group_of_cone.get_permlib_group()), domain_list.begin(), domain_list.end());
 
-   Array<hash_set<int>> induced_orbits(olist.size()); 
+   Array<hash_set<Int>> induced_orbits(olist.size()); 
    auto ioit (entire(induced_orbits));
    for (auto orbit = olist.begin(); orbit != olist.end(); ++orbit, ++ioit) {
-      hash_set<int> one_orbit;
+      hash_set<Int> one_orbit;
       for (auto orb_it=(*orbit)->begin(); orb_it != (*orbit)->end(); ++orb_it) {
          one_orbit += index_of.at(*orb_it); // this will throw if *orb_it isn't in the orbit already, which means the action would be undefined
       }
@@ -232,22 +231,22 @@ orbits_of_induced_action_impl(perl::Object action, const DomainContainer& contai
 } // end anonymous namespace
 
 
-Array<hash_set<int>> orbits_of_induced_action_incidence(perl::Object action, const IncidenceMatrix<>& container) {
-   return orbits_of_induced_action_impl<Set<int>, IncidenceMatrix<>, SetOfIndicesAction<permlib::Permutation>>(action, container);
+Array<hash_set<Int>> orbits_of_induced_action_incidence(BigObject action, const IncidenceMatrix<>& container) {
+   return orbits_of_induced_action_impl<Set<Int>, IncidenceMatrix<>, SetOfIndicesAction<permlib::Permutation>>(action, container);
 }
 
 template<typename Scalar, typename MatrixTop>
-Array<hash_set<int>> orbits_of_coordinate_action(perl::Object action, const GenericMatrix<MatrixTop, Scalar>& container) {
+Array<hash_set<Int>> orbits_of_coordinate_action(BigObject action, const GenericMatrix<MatrixTop, Scalar>& container) {
    return orbits_of_induced_action_impl<Vector<Scalar>, MatrixTop, CoordinateAction<permlib::Permutation, Scalar>>(action, container.top());
 }
 
 
 // wrapper for orbits_in_orbit_order_impl, returns ListReturn instead of Pair
 template <typename MatrixTop, typename Scalar>
-perl::ListReturn orbits_in_orbit_order(perl::Object coordinate_action, const GenericMatrix<MatrixTop, Scalar>& mat)
+ListReturn orbits_in_orbit_order(BigObject coordinate_action, const GenericMatrix<MatrixTop, Scalar>& mat)
 {
    const auto sub_result(orbits_in_orbit_order_impl(coordinate_action, mat));
-   perl::ListReturn result;
+   ListReturn result;
    result << sub_result.first
           << sub_result.second;
    return result;  
@@ -256,7 +255,8 @@ perl::ListReturn orbits_in_orbit_order(perl::Object coordinate_action, const Gen
 
 // test whether one vector is in the orbit of another vector (coordinate action)
 template <typename Scalar>
-bool are_in_same_orbit(perl::Object action, const Vector<Scalar>& vec1, const Vector<Scalar>& vec2) {
+bool are_in_same_orbit(BigObject action, const Vector<Scalar>& vec1, const Vector<Scalar>& vec2)
+{
    typedef permlib::OrbitSet<permlib::Permutation,Vector<Scalar>> VecOrbit;
    boost::shared_ptr<VecOrbit> o(new VecOrbit());
 
@@ -277,32 +277,35 @@ bool are_in_same_orbit(perl::Object action, const Vector<Scalar>& vec1, const Ve
 
 
 // compute all group elements
-Array<Array<int>>
-all_group_elements(perl::Object action) {
-   return Array<Array<int>>(all_group_elements_impl(group_from_perl_action(action)));
+Array<Array<Int>>
+all_group_elements(BigObject action)
+{
+   return Array<Array<Int>>(all_group_elements_impl(group_from_perl_action(action)));
 } 
 
 
 /* stabilizer computations */    
 
-perl::Object stabilizer_of_set (perl::Object action, const Set<int>& set) {
+BigObject stabilizer_of_set(BigObject action, const Set<Int>& set)
+{
    PermlibGroup permlib_group = group_from_perl_action(action);
    PermlibGroup permlib_stab(permlib_group.setwise_stabilizer(set));
-   perl::Object stab = perl_group_from_group(permlib_stab);
+   BigObject stab = perl_group_from_group(permlib_stab);
    stab.set_name("set stabilizer");
    stab.set_description() << "Stabilizer of " << set << endl;
    return stab;
 }
 
-template<typename Scalar>
-perl::Object stabilizer_of_vector (perl::Object action, const Vector<Scalar>& vec) {
-   const int deg = action.give("DEGREE");
+template <typename Scalar>
+BigObject stabilizer_of_vector (BigObject action, const Vector<Scalar>& vec)
+{
+   const Int deg = action.give("DEGREE");
    if (deg != vec.size()-1) {
       throw std::runtime_error("stabilizer_of_vector: the dimension of the vector must be equal to the degree of the group!");
    }
    PermlibGroup permlib_group = group_from_perl_action(action);
    PermlibGroup permlib_stab(permlib_group.vector_stabilizer(vec));
-   perl::Object stab = perl_group_from_group(permlib_stab);
+   BigObject stab = perl_group_from_group(permlib_stab);
    stab.set_name("vector stabilizer");
    stab.set_description() << "Stabilizer of " << vec << endl;
    return stab;
