@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2015
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -22,30 +23,27 @@
 
 namespace polymake { namespace matroid {
 
-Array< Set <int> > bases_from_matroid_polytope(const Matrix<Rational>& verts)
+Array<Set<Int>> bases_from_matroid_polytope(const Matrix<Rational>& verts)
 {
-  Array<Set<int>> bases(verts.rows());
+  Array<Set<Int>> bases(verts.rows());
 
-  for (int i=0;i<verts.rows();++i) {
-    Set<int> b;
-    for (int j=1;j<verts.cols();++j)
-      if (verts(i,j)!=0) b.insert(j-1);
-    bases[i]=b;
+  for (Int i = 0; i < verts.rows(); ++i) {
+     Set<Int> b;
+     for (Int j = 1; j < verts.cols(); ++j)
+        if (verts(i, j) != 0) b.insert(j-1);
+     bases[i] = b;
   }
 
   return bases;
 }
 
-
-perl::Object matroid_from_matroid_polytope(perl::Object p)
+BigObject matroid_from_matroid_polytope(BigObject p)
 {
-  perl::Object m("Matroid");
-  m.take("BASES") << bases_from_matroid_polytope(p.give("VERTICES"));
-  const int n_elements = p.call_method("AMBIENT_DIM");
-  m.take("N_ELEMENTS") << n_elements;
-  m.take("POLYTOPE") << p;
-
-  return m;
+  const Int n_elements = p.call_method("AMBIENT_DIM");
+  return BigObject("Matroid",
+                   "BASES", bases_from_matroid_polytope(p.give("VERTICES")),
+                   "N_ELEMENTS", n_elements,
+                   "POLYTOPE", p);
 }
 
 Function4perl(&bases_from_matroid_polytope, "bases_from_matroid_polytope");
