@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -21,19 +22,16 @@
 #include "polymake/EquivalenceRelation.h"
 
 namespace polymake { namespace matroid {
-namespace {
 
-Array<Set<int>> connected_components_from_circuits(const Set<Set<int>>& circuits, const int n) //cast Array to Set, sorting the sets
+Array<Set<Int>> connected_components_from_circuits(const Set<Set<Int>>& circuits, const Int n) //cast Array to Set, sorting the sets
 {
    EquivalenceRelation components(n);
    for(auto c : circuits) {
       components.merge_classes(c);
    }
-   return Array<Set<int> >(components.equivalence_classes());
+   return Array<Set<Int>>(components.equivalence_classes());
 }
 
-}//end anonymous namespace
-
 Function4perl(&connected_components_from_circuits, "connected_components_from_circuits");
 
 } }
