@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -30,7 +31,7 @@
 namespace polymake { namespace polytope {
 namespace {
 
-typedef Map<int,int> face_map_type;
+typedef Map<Int, Int> face_map_type;
 
 template <typename IMatrix>
 face_map_type count(const Graph<>& G, const GenericIncidenceMatrix<IMatrix>& I)
@@ -40,13 +41,13 @@ face_map_type count(const Graph<>& G, const GenericIncidenceMatrix<IMatrix>& I)
    // Each adjacent pair of edges spans a 2-face.  In order to avoid double counting we enumerate adjacent pairs of
    // edges such that the middle vertex is maximal among all vertices of that 2-face.
    // The to_nodes in the out_edge_list come in ascending order.
-   for (int n1=G.nodes()-1; n1>=0; --n1)
-      for (auto n2=G.out_edges(n1).begin(); !n2.at_end() && n2.to_node()<n1; ++n2) {
-         for (auto n3=G.out_edges(n1).begin(); n3.to_node()<n2.to_node(); ++n3) {
-            const Set<int> facets_thru_all3(I.col(n1)*I.col(n2.to_node())*I.col(n3.to_node()));
+   for (Int n1 = G.nodes()-1; n1 >= 0; --n1)
+      for (auto n2 = G.out_edges(n1).begin(); !n2.at_end() && n2.to_node() < n1; ++n2) {
+         for (auto n3 = G.out_edges(n1).begin(); n3.to_node()<n2.to_node(); ++n3) {
+            const Set<Int> facets_thru_all3(I.col(n1)*I.col(n2.to_node())*I.col(n3.to_node()));
 
             // compute the 2-face spanned by n1, n2.to_node() and n3.to_node()
-            const Set<int> this_face=accumulate(rows(I.minor(facets_thru_all3,All)), operations::mul());
+            const Set<Int> this_face=accumulate(rows(I.minor(facets_thru_all3,All)), operations::mul());
 
             if (this_face.back()==n1)        // n1 largest vertex in 2-face; this avoids double counting
                ++face_map[this_face.size()];
@@ -57,14 +58,14 @@ face_map_type count(const Graph<>& G, const GenericIncidenceMatrix<IMatrix>& I)
 }
 } // end anonymous namespace
 
-void two_face_sizes_simple(perl::Object p)
+void two_face_sizes_simple(BigObject p)
 {
    const Graph<> G=p.give("GRAPH.ADJACENCY");
    const IncidenceMatrix<> I=p.give("VERTICES_IN_FACETS");
    p.take("TWO_FACE_SIZES") << count(G,I);
 }
 
-void subridge_sizes_simple(perl::Object p)
+void subridge_sizes_simple(BigObject p)
 {
    const Graph<> G=p.give("DUAL_GRAPH.ADJACENCY");
    const IncidenceMatrix<> I=p.give("VERTICES_IN_FACETS");
