@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -23,21 +24,20 @@
 namespace polymake { namespace polytope {
 
 template <typename Scalar>
-perl::Object conv(const Array<perl::Object>& pp_in)
+BigObject conv(const Array<BigObject>& pp_in)
 {
-   auto p_in=entire(pp_in);
+   auto p_in = entire(pp_in);
 
    if (p_in.at_end()) throw std::runtime_error("empty input");
 
-   ListMatrix< Vector<Scalar> > Points=p_in->give("VERTICES | POINTS");
-   ListMatrix< Vector<Scalar> > LinSpace=p_in->give("LINEALITY_SPACE");
+   ListMatrix<Vector<Scalar>> Points = p_in->give("VERTICES | POINTS");
+   ListMatrix<Vector<Scalar>> LinSpace = p_in->give("LINEALITY_SPACE");
 
-   perl::Object p_out(perl::ObjectType::construct<Scalar>("Polytope"));
-   std::string descr_names=p_in->name();
+   std::string descr_names = p_in->name();
 
    while (! (++p_in).at_end()) {
-      const Matrix<Scalar> V=p_in->give("VERTICES | POINTS");
-      const Matrix<Scalar> L=p_in->give("LINEALITY_SPACE");
+      const Matrix<Scalar> V = p_in->give("VERTICES | POINTS");
+      const Matrix<Scalar> L = p_in->give("LINEALITY_SPACE");
       if (V.cols() == Points.cols())
          Points /= V;
       else
@@ -46,13 +46,14 @@ perl::Object conv(const Array<perl::Object>& pp_in)
          LinSpace /= L;
       else
          throw std::runtime_error("conv - LinSpace dimension mismatch");
-      descr_names+=", ";
-      descr_names+=p_in->name();
+      descr_names += ", ";
+      descr_names += p_in->name();
    }
 
+   BigObject p_out("Polytope", mlist<Scalar>(),
+                   "INPUT_LINEALITY", LinSpace,
+                   "POINTS", Points);
    p_out.set_description() << "Convex hull of polytopes " << descr_names << endl;
-   p_out.take("INPUT_LINEALITY")<<LinSpace;
-   p_out.take("POINTS") << Points;
    return p_out;
 }
 
