@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -18,49 +19,46 @@
 #include "polymake/Rational.h"
 #include "polymake/SparseMatrix.h"
 
-
 namespace polymake { namespace polytope {
 
 template<typename Scalar>
-perl::Object create_delpezzo(const int d, const Scalar& s, const bool pseudo_flag)
+BigObject create_delpezzo(const Int d, const Scalar& s, const bool pseudo_flag)
 {
    if (d < 1)
       throw std::runtime_error("del_pezzo : dimension d >= 1 required");
 
-   if (d > std::numeric_limits<int>::digits-1)
-      throw std::runtime_error("del_pezzo: in this dimension the number of facets exceeds the machine int size ");
+   if (size_t(d) >= sizeof(Int)*8-1)
+      throw std::runtime_error("del_pezzo: in this dimension the number of facets exceeds the machine Int size");
 
    if (s <= zero_value<Scalar>())
       throw std::runtime_error("del_pezzo : scale > 0 required");
 
-   const int n_vertices= (pseudo_flag) ? 2*d+1 : 2*d+2;
-
-   perl::Object p(perl::ObjectType::construct<Scalar>("Polytope"));
-   p.set_description() << "del-pezzo-polytope of dimension " << d << endl;
+   const Int n_vertices = pseudo_flag ? 2*d+1 : 2*d+2;
 
    SparseMatrix<Scalar> V((s*unit_matrix<Scalar>(d)) / ((-s)*unit_matrix<Scalar>(d)) / (s*ones_vector<Scalar>(d)));
-   if (!pseudo_flag) V /= ((-s)*ones_vector<Scalar>(d));
+   if (!pseudo_flag) V /= -s*ones_vector<Scalar>(d);
    V = ones_vector<Scalar>() | V;
 
-   p.take("CONE_AMBIENT_DIM") << d+1;
-   p.take("CONE_DIM") << d+1;
-   p.take("N_VERTICES") << n_vertices;
-   p.take("VERTICES") << V;
-   p.take("BOUNDED") << true;
-   p.take("CENTERED") << true;
-   p.take("FEASIBLE") << true;
-
+   BigObject p("Polytope", mlist<Scalar>(),
+               "CONE_AMBIENT_DIM", d+1,
+               "CONE_DIM", d+1,
+               "N_VERTICES", n_vertices,
+               "VERTICES", V,
+               "BOUNDED", true,
+               "CENTERED", true,
+               "FEASIBLE", true);
+   p.set_description() << "del-pezzo-polytope of dimension " << d << endl;
    return p;
 }
 
 template<typename Scalar>
-perl::Object delpezzo(const int d, const Scalar& s)
+BigObject delpezzo(const Int d, const Scalar& s)
 {
    return create_delpezzo(d, s, false);
 }
 
 template<typename Scalar>
-perl::Object pseudo_delpezzo(const int d, const Scalar& s)
+BigObject pseudo_delpezzo(const Int d, const Scalar& s)
 {
    return create_delpezzo(d, s, true);
 }
