@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2016
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -15,7 +16,7 @@
 */
 
 #include "polymake/client.h"
-#include "polymake/polytope/hypersimplex.h"
+#include "polymake/polytope/matroid_polytopes.h"
 #include "polymake/Rational.h"
 #include "polymake/Matrix.h"
 #include "polymake/Array.h"
@@ -23,17 +24,18 @@
 #include "polymake/internal/PlainParser.h"
 #include "polymake/SparseMatrix.h"
 #include "polymake/IncidenceMatrix.h"
+#include "polymake/Polynomial.h"
 
 namespace polymake { namespace polytope {
 
-perl::Object hypersimplex(int k, int d, perl::OptionSet options)
+BigObject hypersimplex(Int k, Int d, OptionSet options)
 {
    if (d < 2)
       throw std::runtime_error("hypersimplex: dimension >= 2 required");
    if (k <= 0 || k >= d)
       throw std::runtime_error("hypersimplex: 0 < k < d required");
 
-   perl::Object p("Polytope<Rational>");
+   BigObject p("Polytope<Rational>");
    p.set_description() << "(" << k << "," << d << ")-hypersimplex" << endl;
 
    p.take("CONE_AMBIENT_DIM") << d+1;
@@ -41,7 +43,7 @@ perl::Object hypersimplex(int k, int d, perl::OptionSet options)
    p.take("BOUNDED") << true;
 
    // we already know the number of vertices
-   const int n(Integer::binom(d,k));
+   const Int n(Integer::binom(d,k));
    p.take("N_VERTICES") << n;
 
    const bool group_flag = options["group"];
@@ -50,29 +52,29 @@ perl::Object hypersimplex(int k, int d, perl::OptionSet options)
    bool novif_flag = options["no_vif"];
 
    // if the hypersimplex degenerates to a simplex the formula for the facets below don't work
-   if ( k==1 || k==d-1 ) {
+   if (k == 1 || k == d-1) {
       nof_flag = novif_flag = true;
    }
-   
-   if ( !nov_flag ) {
-      Array<std::string> labels(n);
+
+   if (!nov_flag) {
       std::ostringstream label;
-      int i(0);
+      Array<std::string> labels(n);
+      auto label_it = labels.begin();
       Matrix<Rational> Vertices(n,d+1);
-      Rows< Matrix<Rational> >::iterator v=rows(Vertices).begin();
-      for (auto s=entire(all_subsets_of_k(range(0,d-1),k)); !s.at_end(); ++s, ++v,++i) {
+      auto v = rows(Vertices).begin();
+      for (auto s=entire(all_subsets_of_k(range(0,d-1),k)); !s.at_end(); ++s, ++v, ++label_it) {
          (*v)[0]=1;
-         v->slice(1).slice(*s).fill(1);
+         v->slice(range_from(1)).slice(*s).fill(1);
          label.str("");
          wrap(label) << *s;
-         labels[i] = label.str();
+         *label_it = label.str();
       }
       p.take("VERTEX_LABELS") << labels;
       p.take("VERTICES") << Vertices;
 
-      if ( !novif_flag ) {
+      if (!novif_flag) {
          IncidenceMatrix<> VIF(2*d,n);
-         for (int i=0; i<d; ++i) {
+         for (Int i = 0; i < d; ++i) {
             VIF.row(2*i)  = indices(attach_selector(Vertices.col(i+1), operations::non_zero()));
             VIF.row(2*i+1)= indices(attach_selector(Vertices.col(i+1), operations::is_zero()));
          }
@@ -80,12 +82,12 @@ perl::Object hypersimplex(int k, int d, perl::OptionSet options)
       }
    }
 
-   if ( nov_flag && !novif_flag ) {
+   if (nov_flag && !novif_flag) {
       IncidenceMatrix<> VIF(2*d,n);
-      int j(0);
-      for (Entire<Subsets_of_k<const sequence&> >::const_iterator s=entire(all_subsets_of_k(range(0,d-1),k)); !s.at_end(); ++s, ++j) {
-         for (int i=0; i<d; ++i) {
-            if (Set<int>(*s).contains(i)) {
+      Int j = 0;
+      for (auto s = entire(all_subsets_of_k(range(0,d-1), k)); !s.at_end(); ++s, ++j) {
+         for (Int i = 0; i < d; ++i) {
+            if (Set<Int>(*s).contains(i)) {
                VIF.row(2*i) += j;
             } else {
                VIF.row(2*i+1) += j;
@@ -99,8 +101,8 @@ perl::Object hypersimplex(int k, int d, perl::OptionSet options)
    // the facets are not unique
    if ( !nof_flag ) {
       SparseMatrix<Rational> F(2*d,d+1);
-      Rows< SparseMatrix<Rational> >::iterator f=rows(F).begin();   
-      for (int i=1; i<=d; ++i) { // Facet 2*i and Facet 2*i+1 are parallel  
+      Rows< SparseMatrix<Rational> >::iterator f=rows(F).begin();
+      for (Int i = 1; i <= d; ++i) { // Facet 2*i and Facet 2*i+1 are parallel
          (*f)[0]=1;
          (*f)[i]=-1;
          ++f;
@@ -110,55 +112,54 @@ perl::Object hypersimplex(int k, int d, perl::OptionSet options)
       }
       Matrix<Rational> A(1,d+1);
       A.row(0)=ones_vector<Rational>(d+1);
-      A.row(0)[0]=-k;   
+      A.row(0)[0]=-k;
       p.take("FACETS") << F;
       p.take("AFFINE_HULL") << A;
    }
+   
+   if(!nof_flag || !nov_flag)
+	   p.take("EHRHART_POLYNOMIAL") << ehrhart_polynomial_hypersimplex(k,d);
 
    // generate the combinatorial symmetry group on the vertices
-   if ( group_flag ) {
-      Array<Array<int>> gens(2);
-      Array<int> gen{sequence(0,d)};
-      gen[0]=1;
-      gen[1]=0;
-      gens[0]=gen;
-         
-      gen[0]=d-1;
-      for (int j=1; j<=d-1; ++j) {
-         gen[j]=j-1; 
+   if (group_flag) {
+      Array<Array<Int>> gens(2);
+      Array<Int> gen{sequence(0,d)};
+      gen[0] = 1;
+      gen[1] = 0;
+      gens[0] = gen;
+
+      gen[0] = d-1;
+      for (Int j = 1; j <= d-1; ++j) {
+         gen[j] = j-1;
       }
-      gens[1]=gen;
+      gens[1] = gen;
 
-      perl::Object a("group::PermutationAction");
-      a.take("GENERATORS") << gens;
-
-      perl::Object g("group::Group");
+      BigObject a("group::PermutationAction", "GENERATORS", gens);
+      BigObject g("group::Group", "fullCombinatorialGroupOnCoords");
       g.set_description() << "full combinatorial group on coordinates of " <<  "(" << k << "," << d << ")-hypersimplex" << endl;
-      g.set_name("fullCombinatorialGroupOnCoords");
-
       p.take("GROUP") << g;
       p.take("GROUP.COORDINATE_ACTION") << a;
    }
-
+   
    return p;
 }
 
-Set<int> matroid_indices_of_hypersimplex_vertices(perl::Object m)
+Set<Int> matroid_indices_of_hypersimplex_vertices(BigObject m)
 {
-   const Array< Set<int> > bases=m.give("BASES");
-   const int n=m.give("N_ELEMENTS");
-   const int d=m.give("RANK");
-   Set<int> set;
+   const Array<Set<Int>> bases=m.give("BASES");
+   const Int n = m.give("N_ELEMENTS");
+   const Int d = m.give("RANK");
+   Set<Int> set;
    for (const auto& b : bases) {
-      int sum(0);
-      int temp_d(d);
-      int temp(0);
+      Int sum = 0;
+      Int temp_d = d;
+      Int temp = 0;
       for (const auto& i : b) {
-         if (temp_d==d && i!=0)
-            sum += int(Integer::binom(n-1, d-1));
+         if (temp_d == d && i != 0)
+            sum += Int(Integer::binom(n-1, d-1));
          --temp_d;
-         for (int k=1; k <= i-temp-1; ++k)
-            sum += int(Integer::binom(n-temp-1-k, temp_d));
+         for (Int k = 1; k <= i-temp-1; ++k)
+            sum += Int(Integer::binom(n-temp-1-k, temp_d));
          temp=i;
       }
       set += sum;
@@ -166,9 +167,8 @@ Set<int> matroid_indices_of_hypersimplex_vertices(perl::Object m)
    return set;
 }
 
-
 UserFunction4perl("# @category Producing a polytope from scratch"
-                  "# Produce the hypersimplex &Delta;(//k//,//d//), that is the the convex hull of all 0/1-vector in R<sup>//d//</sup>"
+                  "# Produce the hypersimplex $ Δ(k,d) $, that is the the convex hull of all 0/1-vector in $ R^d $"
                   "# with exactly //k// 1s."
                   "# Note that the output is never full-dimensional."
                   "# @param Int k number of 1s"
