@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2015
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -22,6 +23,7 @@
 #include "polymake/Matrix.h"
 #include "polymake/Array.h"
 #include "polymake/Vector.h"
+#include "polymake/common/labels.h"
 #include "polymake/polytope/bisector.h"
 
 namespace polymake { namespace polytope {
@@ -33,7 +35,7 @@ void nearest_vertex(const GenericVector<VectorTop, Rational>& F, const Vector<Ra
 {
    Rational r=F*F_normal;
    if (r>0) {
-      r=F*Fb / r;
+      r = F*Fb/r;
       if (r<min_ratio) min_ratio=r;
    }
 }
@@ -42,14 +44,14 @@ template <typename SetTop>
 Vector<Rational>
 compute_new_vertex(const Matrix<Rational>& Facets, const Matrix<Rational>& Vertices,
                    const Vector<Rational>& Vb, const IncidenceMatrix<>& VIF,
-                   const Graph<>& DG, const GenericSet<SetTop>& stack_facets, int sf, const Rational& lift_factor)
+                   const Graph<>& DG, const GenericSet<SetTop>& stack_facets, Int sf, const Rational& lift_factor)
 {
    const Vector<Rational> Fb=average(rows(Vertices.minor(VIF[sf],All)));
    Vector<Rational> F_normal=Facets[sf];
-   F_normal[0] = -(F_normal.slice(1)*Fb.slice(1));           // facet normal thru facet barycenter
+   F_normal[0] = -(F_normal.slice(range_from(1)) * Fb.slice(range_from(1)));      // facet normal thru facet barycenter
    Rational min_ratio=(Vb-Fb)*F_normal/sqr(F_normal);        // furthest position: mirror opposite of VERTEX_BARYCENTER
 
-   for (Entire<Graph<>::adjacent_node_list>::const_iterator nb=entire(DG.adjacent_nodes(sf)); !nb.at_end();  ++nb) {
+   for (auto nb=entire(DG.adjacent_nodes(sf)); !nb.at_end();  ++nb) {
       if (stack_facets.top().contains(*nb))
          nearest_vertex(bisector(Facets[sf], Facets[*nb], Vertices[ (VIF[sf] * VIF[*nb]).front() ]),
                         F_normal, Fb, min_ratio);
@@ -62,7 +64,7 @@ compute_new_vertex(const Matrix<Rational>& Facets, const Matrix<Rational>& Verti
 } // end unnamed namespace
 
 template <typename TSet>
-perl::Object stack(perl::Object p_in, const GenericSet<TSet>& stack_facets, perl::OptionSet options)
+BigObject stack(BigObject p_in, const GenericSet<TSet>& stack_facets, OptionSet options)
 {
    const bool bounded = p_in.give("BOUNDED");
    if (!bounded)
@@ -77,33 +79,33 @@ perl::Object stack(perl::Object p_in, const GenericSet<TSet>& stack_facets, perl
       if (lift_factor<=0 || lift_factor>=1)
          throw std::runtime_error("lift factor must be between 0 and 1");
    }
-   const bool relabel=!options["no_labels"],
-      noc=options["no_coordinates"];
-   const int dim=p_in.give("COMBINATORIAL_DIM");
-   if (dim<=2)
+   const bool relabel = !options["no_labels"],
+                  noc = options["no_coordinates"];
+   const Int dim = p_in.give("COMBINATORIAL_DIM");
+   if (dim <= 2)
       throw std::runtime_error("dimension too low to distinguish between simpliciality and cubicality");
 
-   const bool simplicial=p_in.give("SIMPLICIAL"),
-      cubical=p_in.give("CUBICAL");
+   const bool simplicial = p_in.give("SIMPLICIAL"),
+                 cubical = p_in.give("CUBICAL");
    if (!simplicial && !cubical)
       throw std::runtime_error("polytope neither simplicial nor cubical");
 
-   const IncidenceMatrix<> VIF=p_in.give("VERTICES_IN_FACETS");
-   const int n_vertices=VIF.cols(), n_facets=VIF.rows();
+   const IncidenceMatrix<> VIF = p_in.give("VERTICES_IN_FACETS");
+   const Int n_vertices = VIF.cols(), n_facets=VIF.rows();
 
    if (stack_facets.top().empty())
       throw std::runtime_error("stack: no facets to stack specified");
    if (stack_facets.top().front() < 0 || stack_facets.top().back() >= n_facets)
       throw std::runtime_error("facet numbers out of range");
 
-   perl::Object p_out("Polytope<Rational>");
-   if (std::is_same<TSet, Set<int> >::value)
+   BigObject p_out("Polytope<Rational>");
+   if (std::is_same<TSet, Set<Int>>::value)
       p_out.set_description() << p_in.name() << " with facets " << stack_facets << " stacked" << endl;
 
-   const int n_stack_facets=stack_facets.top().size();
+   const Int n_stack_facets = stack_facets.top().size();
 
-   const int n_vertices_out= n_vertices + n_stack_facets * (simplicial ?     1 : 1 << dim-1),
-      n_facets_out  = n_facets   + n_stack_facets * (simplicial ? dim-1 : 2*(dim-1));
+   const Int n_vertices_out = n_vertices + n_stack_facets * (simplicial ?     1 : 1L << dim-1),
+              n_facets_out  = n_facets   + n_stack_facets * (simplicial ? dim-1 : 2*(dim-1));
    p_out.take("COMBINATORIAL_DIM") << dim;
    p_out.take("N_VERTICES") << n_vertices_out;
 
@@ -114,9 +116,9 @@ perl::Object stack(perl::Object p_in, const GenericSet<TSet>& stack_facets, perl
 
    std::vector<std::string> labels, facet_labels;
    if (relabel) {
-      labels.resize(n_vertices_out);  facet_labels.resize(n_facets);
-      read_labels(p_in, "VERTEX_LABELS", labels);
-      read_labels(p_in, "FACET_LABELS", facet_labels);
+      labels.resize(n_vertices_out);
+      common::read_labels(p_in, "VERTEX_LABELS", select(labels, sequence(0, n_vertices)));
+      facet_labels = common::read_labels(p_in, "FACET_LABELS", n_facets);
    }
 
    Matrix<Rational> Vertices, Facets, Vertices_out;
@@ -129,27 +131,26 @@ perl::Object stack(perl::Object p_in, const GenericSet<TSet>& stack_facets, perl
    }
 
    if (simplicial) {
-      int nv=n_vertices;     // new vertex
-      for (auto sf=entire(stack_facets.top());  !sf.at_end();  ++sf, ++nv) {
+      Int new_vertex = n_vertices;     // new vertex
+      for (auto sf=entire(stack_facets.top());  !sf.at_end();  ++sf, ++new_vertex) {
          // new facet = old facet - one of its vertices + new vertex
-         for (Subsets_less_1<const IncidenceMatrix<>::const_row_type&>::const_iterator
-                 ridges=entire(all_subsets_less_1(VIF[*sf]));  !ridges.at_end();  ++ridges, ++new_facet) {
-            *new_facet=*ridges;
-            new_facet->push_back(nv);
+         for (auto ridges = entire(all_subsets_less_1(VIF[*sf]));  !ridges.at_end();  ++ridges, ++new_facet) {
+            *new_facet = *ridges;
+            new_facet->push_back(new_vertex);
          }
       }
 
       if (!noc) {
-         const Graph<> DG=p_in.give("DUAL_GRAPH.ADJACENCY");
-         Rows< Matrix<Rational> >::iterator nv=rows(Vertices_out).begin()+n_vertices;
-         for (auto sf=entire(stack_facets.top());  !sf.at_end();  ++sf, ++nv) {
-            *nv=compute_new_vertex(Facets,Vertices,Vb,VIF,DG,stack_facets,*sf,lift_factor);
+         const Graph<> DG = p_in.give("DUAL_GRAPH.ADJACENCY");
+         auto nv = rows(Vertices_out).begin() + n_vertices;
+         for (auto sf = entire(stack_facets.top());  !sf.at_end();  ++sf, ++nv) {
+            *nv = compute_new_vertex(Facets, Vertices, Vb, VIF, DG, stack_facets, *sf, lift_factor);
          }
       }
 
       if (relabel) {
-         std::vector<std::string>::iterator nl=labels.begin()+n_vertices;
-         for (auto sf=entire(stack_facets.top());  !sf.at_end();  ++sf, ++nl)
+         auto nl = labels.begin() + n_vertices;
+         for (auto sf = entire(stack_facets.top());  !sf.at_end();  ++sf, ++nl)
             *nl = "f(" + facet_labels[*sf] + ')';
       }
 
@@ -157,20 +158,20 @@ perl::Object stack(perl::Object p_in, const GenericSet<TSet>& stack_facets, perl
       // cubical
       const Graph<> DG=p_in.give("DUAL_GRAPH.ADJACENCY");
 
-      int first_new_vertex=n_vertices;
-      const int vertices_per_facet=1<<dim-1;
-      std::vector<int> new_neighbors(n_vertices);
+      Int first_new_vertex = n_vertices;
+      const Int vertices_per_facet = 1L<<dim-1;
+      std::vector<Int> new_neighbors(n_vertices);
 
       for (auto sf=entire(stack_facets.top());  !sf.at_end();  ++sf) {
          *new_facet=sequence(first_new_vertex, vertices_per_facet);
          ++new_facet;
          copy_range(entire(sequence(first_new_vertex, vertices_per_facet)), select(new_neighbors, VIF[*sf]).begin());
 
-         for (Graph<>::adjacent_node_list::const_iterator nb=DG.adjacent_nodes(*sf).begin();
+         for (auto nb=DG.adjacent_nodes(*sf).begin();
               !nb.at_end();  ++nb, ++new_facet) {
-            const Set<int> ridge=VIF[*sf] * VIF[*nb];
+            const Set<Int> ridge = VIF[*sf] * VIF[*nb];
             *new_facet = ridge;
-            *new_facet += assure_ordered(select(new_neighbors,ridge));
+            *new_facet += assure_ordered(select(new_neighbors, ridge));
          }
 
          if (!noc) {
@@ -180,7 +181,7 @@ perl::Object stack(perl::Object p_in, const GenericSet<TSet>& stack_facets, perl
          }
 
          if (relabel) {
-            for (IncidenceMatrix<>::const_row_type::const_iterator v=VIF[*sf].begin(); !v.at_end();  ++v)
+            for (auto v=VIF[*sf].begin(); !v.at_end(); ++v)
                labels[new_neighbors[*v]] = "f(" + facet_labels[*sf] + ")-" + labels[*v];
          }
          first_new_vertex += vertices_per_facet;
@@ -200,25 +201,25 @@ perl::Object stack(perl::Object p_in, const GenericSet<TSet>& stack_facets, perl
    return p_out;
 }
 
-perl::Object stack(perl::Object p_in, const pm::all_selector&, perl::OptionSet options)
+BigObject stack(BigObject p_in, const pm::all_selector&, OptionSet options)
 {
-   const int n_facets=p_in.give("N_FACETS");
-   perl::Object p_out=stack(p_in, sequence(0,n_facets), options);
+   const Int n_facets = p_in.give("N_FACETS");
+   BigObject p_out = stack(p_in, sequence(0,n_facets), options);
    p_out.set_description() << p_in.name() << " with all facets stacked" << endl;
    return p_out;
 }
 
-perl::Object stack(perl::Object p_in, int facet, perl::OptionSet options)
+BigObject stack(BigObject p_in, Int facet, OptionSet options)
 {
-   perl::Object p_out=stack(p_in, scalar2set(facet), options);
+   BigObject p_out = stack(p_in, scalar2set(facet), options);
    p_out.set_description() << p_in.name() << " with facet " << facet << " stacked" << endl;
    return p_out;
 }
 
-perl::Object stack(perl::Object p_in, const Array<int>& facets, perl::OptionSet options)
+BigObject stack(BigObject p_in, const Array<Int>& facets, OptionSet options)
 {
-   Set<int> stack_facets;
-   for (Entire< Array<int> >::const_iterator fi = entire(facets); !fi.at_end(); ++fi)
+   Set<Int> stack_facets;
+   for (auto fi = entire(facets); !fi.at_end(); ++fi)
       stack_facets += *fi;
 
    if (stack_facets.size() != facets.size())
