@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -21,7 +21,7 @@
 #include "polymake/Vector.h"
 #include "polymake/Graph.h"
 #include "polymake/Rational.h"
-#include "polymake/RandomSpherePoints.h"
+#include "polymake/RandomPoints.h"
 #include "polymake/graph/Lattice.h"
 #include "polymake/graph/Decoration.h"
 #include <cmath>
