@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2015
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -20,14 +21,14 @@
 
 namespace polymake { namespace polytope {
 
-template <typename Scalar>
-Array<int> vertex_point_map(const Matrix<Scalar>& verts, const Matrix<Scalar>& points)
+template <typename Matrix1, typename Matrix2, typename Scalar>
+Array<Int> vertex_point_map(const GenericMatrix<Matrix1, Scalar>& verts, const GenericMatrix<Matrix2, Scalar>& points)
 {
-  const int n=verts.rows();
-  Array<int> vpm(n,-1);
+  const Int n = verts.rows();
+  Array<Int> vpm(n, -1);
 
-  for (int i=0; i<n; ++i) {
-    for (int j=0; j<points.rows(); ++j) 
+  for (Int i = 0; i < n; ++i) {
+    for (Int j = 0; j < points.rows(); ++j) 
       if (verts.row(i)==points.row(j)) {
         vpm[i]=j;
         break;
