@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2022
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -39,12 +40,10 @@ sub application {
             add Core::Application($new_app);
          }
       } else {
-         $new_app=add Core::Application($new_app) unless ref($new_app);
-         if (defined($application)) {
-            return if $application == $new_app;
-            readwrite($application);
-         }
-         $application=$new_app;
+         $new_app = add Core::Application($new_app) unless ref($new_app);
+         return if $application == $new_app;
+         readonly_off($application);
+         $application = $new_app;
          readonly($application);
       }
    } else {
@@ -55,7 +54,7 @@ sub application {
 
 #################################################################################
 sub include {
-   my $rc=$application->include_rules(@_);
+   my $rc = $application->include_rules(@_);
    if ($rc != @_) {
       foreach (@_) {
          my ($app, $rulefile)= /^($id_re)::/o ? ($application->used->{$1}, $') : ($application, $_);
@@ -74,8 +73,8 @@ sub include {
 }
 #################################################################################
 sub load {
-   my ($name)=@_;
-   my $filename=$name;
+   my ($name) = @_;
+   my $filename = $name;
    replace_special_paths($filename);
    unless (-f $filename
            or
@@ -83,36 +82,39 @@ sub load {
            -f ($filename .= "." . $application->default_file_suffix)) {
       croak( "no such file: $name" );
    }
-   my $obj=load Core::Object($filename);
-   $obj;
+   load Core::Datafile($filename);
 }
 #################################################################################
 sub save {
-   my ($obj, $filename)=@_;
-   if (!is_object($obj) || !instanceof Core::Object($obj)) {
-      croak( "only polymake Objects can be stored with the function save()" );
-   }
-   if (defined($obj->parent)) {
-      croak( "A sub-object can't be saved without its parent" );
-   }
-   if (!$obj->persistent || defined($filename)) {
-      if (defined $filename) {
+   my ($obj, $filename, %options) = @_;
+   if (instanceof Core::BigObject($obj)) {
+      my ($schema, $datafile);
+      if (defined($filename)) {
+         $schema = delete $options{schema};
          replace_special_paths($filename);
          if ($filename !~ /\.\w+$/) {
             $filename .= "." . $obj->default_file_suffix;
          }
+         $datafile = new Core::Datafile($filename, \%options);
+         $obj->persistent = $datafile unless defined($schema);
+
+      } elsif (defined($datafile = $obj->persistent)) {
+         unless ($obj->changed) {
+            warn_print( "no changes need to be saved" );
+            return;
+         }
 
       } else {
          if (!length($obj->name)) {
-            $obj->name=Core::name_of_arg_var(0);
+            $obj->name = Core::name_of_arg_var(0);
          }
          if (length($obj->name)) {
-            $filename=$obj->name.".".$obj->default_file_suffix;
+            $filename = $obj->name . "." . $obj->default_file_suffix;
             if (-f $filename) {
                if ($Shell->interactive) {
                   print "The file $filename already exists.\n",
                         "Please specify another file name or confirm it to be overwritten:\n";
-                  $filename=$Shell->enter_filename($filename, { prompt => "data file" }) or return;
+                  $filename = $Shell->enter_filename($filename, { prompt => "data file" }) or return;
                } else {
                   croak( "Can't save an object '", $obj->name, "' since the file $filename already exists.\n",
                          "Please specify the explicit file name as the second argument to save() or delete the existing file (unlink \"$filename\")" );
@@ -121,69 +123,34 @@ sub save {
          } else {
             if ($Shell->interactive) {
                print "Please specify the file name for the anonymous ", $obj->type->full_name, " object:\n";
-               $filename=$Shell->enter_filename("", { prompt => "data file" }) or return;
+               $filename = $Shell->enter_filename("", { prompt => "data file" }) or return;
             } else {
-               my $i=1;
-               while (-f ($filename=($obj->name=$obj->type->name."_$i".".".$obj->default_file_suffix))) { ++$i }
+               my $i = 1;
+               while (-f ($filename = $obj->type->name . "_$i" . "." . $obj->default_file_suffix)) { ++$i }
                warn_print( "saving object as $filename" );
             }
          }
+         $datafile = $obj->persistent = new Core::Datafile($filename, \%options);
       }
-      $obj->persistent=new Core::XMLfile($filename);
-      $obj->changed=1;
+      $datafile->save($obj, schema => $schema);
 
-   } elsif (! $obj->changed) {
-      warn_print( "no changes need to be saved" );
-      return;
+   } else {
+      &save_data;
    }
-
-   $obj->save;
-}
-#################################################################################
-sub save_schema {
-   if (@_<2) {
-      die "usage: save_schema(Object or ObjectType, ..., \"filename\"\n";
-   }
-   my $filename=pop;
-   if ($filename !~ /\.\w+$/) {
-      $filename .= ".rng";
-   }
-   replace_special_paths($filename);
-   my $xf=new Core::XMLfile($filename);
-   $xf->save_schema(map {
-           if (is_object($_) && UNIVERSAL::can($_, "type")) {
-              $_->type
-           } else {
-              die "argument ", ref($_) || "'$_'", " is not an object or a type expression\n";
-           }
-        } @_);
 }
 #################################################################################
 sub load_data {
-   my ($filename)=@_;
+   my ($filename) = @_;
    replace_special_paths($filename);
-   my $xf=new Core::XMLfile($filename);
-   scalar($xf->load_data);
+   load Core::Datafile($filename);
 }
 
 sub save_data {
-   my ($data, $filename, $description)=@_;
-   if (!is_object($data)) {
-      croak( "only complex data types can be saved in separate files" );
-   }
-   if (instanceof Core::Object($data)) {
-      croak( "an object of type ", $data->type->full_name, " can't be saved with save_data: please use save() instead" );
-   }
-   if (defined (my $type=UNIVERSAL::can($data, ".type"))) {
-      $type=$type->();
-      if (instanceof Core::PropertyType($type)) {
-         replace_special_paths($filename);
-         my $xf=new Core::XMLfile($filename);
-         $xf->save_data($data, $description);
-         return;
-      }
-   }
-   croak( "can't save ", ref($data), ": it does not belong to any declared property type" );
+   my ($data, $filename, %options) = @_;
+   defined($filename) or croak( "filename missing" );
+   replace_special_paths($filename);
+   my $df = new Core::Datafile($filename, \%options);
+   $df->save($data);
 }
 #################################################################################
 use subs qw(rename unlink mkdir chdir rmdir);
@@ -237,39 +204,34 @@ sub disable_rules {
    $application->disable_rules(@_);
 }
 #################################################################################
-sub set_custom {
-   $application->_set_custom($Prefs->custom, Core::name_of_custom_var(1));
-}
-
-sub reset_custom {
-   $application->_reset_custom($Prefs->custom, Core::name_of_custom_var(0));
-}
-#################################################################################
 sub script {
-   my $name=shift;
+   my $name = shift;
    replace_special_paths($name);
-   my ($code, $full_path, $in_app)=find Core::StoredScript($name);
+   my ($code, $full_path, $in_app) = find Core::StoredScript($name);
    if (defined $code) {
-      local *ARGV=\@_;
+      local *ARGV = \@_;
       &$code;
    } else {
-      local @ARGV=@_;
-
-      # The following awful expression *can't* be broken down into more legible if-else branches
-      # because all localizations have to occur in this block.
-      defined($in_app)
-        ? $in_app != $User::application && (local $User::application=$in_app,
-                                            ref($INC[0]) ? (local $INC[0]=$in_app) : local_unshift(\@INC, $in_app))
-        : (local_save_scalar($User::application),
-           ref($INC[0]) || local_unshift(\@INC, $User::application));
-
-      $name="script" . (defined($in_app) ? ":" : "=") . $full_path;
+      local @ARGV = @_;
+      local if (defined($in_app)) {
+         if ($in_app != $application) {
+            local $application = $in_app;
+            if (ref($INC[0])) {
+               local $INC[0] = $in_app;
+            } else {
+               local unshift @INC, $in_app;
+            }
+         }
+      } elsif (!ref($INC[0])) {
+         local unshift @INC, $application;
+      }
+      $name = "script" . (defined($in_app) ? ":" : "=") . $full_path;
       if (wantarray) {
-         my @ret=do $name;
+         my @ret = do $name;
          die $@ if $@;
          @ret
       } elsif (defined wantarray) {
-         my $ret=do $name;
+         my $ret = do $name;
          die $@ if $@;
          $ret
       } else {
@@ -279,45 +241,55 @@ sub script {
    }
 }
 #################################################################################
-# prepare for custom variables and preferences
+# print boolean values in legible form: true and false instead of 1 and empty string
+# enforce creation of a unique lexical scope with this operation inherited by all nested packages
+use namespaces 'Polymake::User';
+namespaces::memorize_lexical_scope;
+namespaces::intercept_operation(undef, "print", "bool");
+
+#################################################################################
+# prepare custom variables
 
 package Polymake::User::Verbose;
 *Polymake::Verbose::=get_symtab(__PACKAGE__);
 
-push @Core::UserSettings::add_custom_vars,
-sub {
-   my $ch=$Prefs->create_custom("Polymake::User");
+Core::add_settings_callback sub {
+   my ($settings) = @_;
+   my %docs;
 
-   $ch->pkg_help->{__PACKAGE__}=<<'.';
-# The following variables control the display of various informational message classes.
-.
-   declare $credits=1;
-   $ch->add('$credits', <<'.');
+   declare $credits = 1;
+   $settings->add_item("Polymake::User::Verbose::credits", \$credits,
+                       $docs{'$Polymake::User::Verbose::credits'} = <<'.', 0);
 # Display the copyright notices and URLs of third-party software:
 # 0 - never, 1 - at the first use in the current session, 2 - always
 .
-   declare $rules=1;
-   $ch->add('$rules', <<'.');
+   declare $rules = 1;
+   $settings->add_item("Polymake::User::Verbose::rules", \$rules,
+                       $docs{'$Polymake::User::Verbose::rules'} = <<'.', 0);
 # Display the information about the rules:
 # 0 - nothing, 1 - significant failures, 2 - summary and all failed preconditions, 3 - executed rule executed
 .
-   declare $scheduler=0;
-   $ch->add('$scheduler', <<'.');
+   declare $scheduler = 0;
+   $settings->add_item("Polymake::User::Verbose::scheduler", \$scheduler,
+                       $docs{'$Polymake::User::Verbose::scheduler'} = <<'.', 0);
 # Reveal the internals of the rule scheduler:
 # 0 - nothing, 1 - summary and statistics, 2 - initial rule selection,
 # 3 - shortest path search (overwhelming amount of data)
 .
-   declare $cpp=0;
-   $ch->add('$cpp', <<'.');
+   declare $cpp = 0;
+   $settings->add_item("Polymake::User::Verbose::cpp", \$cpp,
+                       $docs{'$Polymake::User::Verbose::cpp'} = <<'.', 0);
 # Tell about the actions of the perl/C++ interface generator:
 # 0 - nothing, 1 - compiler calls and source file updates, 2 - source code generated
 .
-   declare $files=1;
-   $ch->add('$files', <<'.');
+   declare $files = 1;
+   $settings->add_item("Polymake::User::Verbose::files", \$files,
+                       $docs{'$Polymake::User::Verbose::files'} = <<'.', 0);
 # Notify about nontrivial actions during data file processing
 .
-   declare $external=0;
-$ch->add('$external', <<'.');
+   declare $external = 0;
+   $settings->add_item("Polymake::User::Verbose::external", \$external,
+                       $docs{'$Polymake::User::Verbose::external'} = <<'.', 0);
 # Notify about external programs starting in the background
 # (not to be mixed up with credits!)
 .
@@ -325,61 +297,72 @@ $ch->add('$external', <<'.');
    package Polymake::User;
 
    declare @start_applications;
-   $ch->add('@start_applications', <<'.');
+   $settings->add_item("Polymake::User::start_applications", \@start_applications,
+                       $docs{'@start_applications'} = <<'.', 0);
 # Applications to be loaded at the beginning of each interactive or batch session
 .
    declare $default_application;
-   $ch->add('$default_application', <<'.');
+   $settings->add_item("Polymake::User::default_application", \$default_application,
+                       $docs{'$default_application'} = <<'.', 0);
 # Application to start with as the current one
 .
    declare @extensions;
-   $ch->add('@extensions', <<'.', $Core::Customize::state_accumulating);
+   $settings->add_item("Polymake::User::extensions", \@extensions,
+                       $docs{'@extensions'} = <<'.', Core::UserSettings::Item::Flags::accumulating);
 # A list of directories containing imported and/or locally created extensions
-.
-   declare %disabled_extensions;
-   $ch->add('%disabled_extensions', <<'.', $Core::Customize::state_config | $Core::Customize::state_hidden | $Core::Customize::state_noexport);
-# Extensions which could not be configured for given architecture
 .
    declare @lookup_scripts;
-   $ch->add('@lookup_scripts', <<'.', $Core::Customize::state_accumulating);
+   $settings->add_item("Polymake::User::lookup_scripts", \@lookup_scripts,
+                       $docs{'@lookup_scripts'} = <<'.', Core::UserSettings::Item::Flags::accumulating);
 # A list of directories where to look for scripts
 .
-   declare $history_size=200;
-   $ch->add('$history_size', <<'.');
+   declare $init_script;
+   $settings->add_item("Polymake::User::init_script", \$init_script,
+                       $docs{'$init_script'} = <<'.', 0);
+# Script executed at the beginning of every session.
+# If specified without absolute path, looked up in the private settings directory (~/.polymake).
+# The script can perform custom initialization actions, e.g. define simple shortcut commands,
+# amend package lookup paths, or load additional rule files.
+.
+   declare $history_size = 200;
+   $settings->add_item("Polymake::User::history_size", \$history_size,
+                       $docs{'$history_size'} = <<'.', 0);
 # Maximal number of commands stored in the interactive shell's history.
 # If set to undef, history list grows unlimited.
 .
-   declare $history_editor=$ENV{VISUAL} || $ENV{EDITOR} || "vi";
-   $ch->add('$history_editor', <<'.');
+   declare $history_editor = $ENV{VISUAL} || $ENV{EDITOR} || "vi";
+   $settings->add_item("Polymake::User::history_editor", \$history_editor,
+                       $docs{'$history_editor'} = <<'.', 0);
 # Editor for the ''history'' command.
 # Must be a complete shell command. If the temporary file name is expected somewhere in the middle
 # of the arguments, please use the placeholder %f.
 .
-   declare $help_key="_k1";
-$ch->add('$help_key', <<'.');
+   declare $help_key = "_k1";
+   $settings->add_item("Polymake::User::help_key", \$help_key,
+                       $docs{'$help_key'} = <<'.', 0);
 # Key to press for interactive help in the shell.  Defaults to F1.
 .
-   declare $help_delimit=1;
-$ch->add('$help_delimit', <<'.');
+   declare $help_delimit = 1;
+   $settings->add_item("Polymake::User::help_delimit", \$help_delimit,
+                       $docs{'$help_delimit'} = <<'.', 0);
 # Add delimiters for better readability in help text.
 .
 
-   $ch->cleanup;
-
-   # rescue the old-fashioned lookup_applications list
-   if (defined (my $lookup_apps=*lookup_applications{ARRAY})) {
-      push @extensions, @$lookup_apps;
-      $ch->set('@extensions');
-   }
-
    # treat relative paths as starting at $HOME
    s{^(?:~/|(?!/))}{$ENV{HOME}/} for @extensions, @lookup_scripts;
 
    if (!@start_applications) {
-      $ch->set('@start_applications', map { m{apps/([^/]+)/rules} } glob("$InstallTop/apps/*/rules/main.rules"));
+      @start_applications = map { m{apps/([^/]+)/rules} } glob("$InstallTop/apps/*/rules/main.rules");
    }
    if (!$default_application) {
-      $ch->set('$default_application', string_list_index(\@start_applications, "polytope")>=0 ? "polytope" : $start_applications[0]);
+      $default_application = string_list_index(\@start_applications, "polytope") >= 0 ? "polytope" : $start_applications[0];
+   }
+
+   Core::Help::add_activation_callback sub {
+      my ($help) = @_;
+      while (my ($varname, $text) = each %docs) {
+         $help->add(['custom', $varname], $text);
+      }
    }
 };
 
